library IEEE;
-- top_level.vhd: top level architecture for Processor Expert demo
-- Copyright (C) Invea-tech 2007
--
-- $Id: top_level_template.vhd 554 2007-10-05 15:20:45Z pazdera $
--
--

use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

architecture arch_demo of tlv_bare_ifc is

   -- SPI address decoder driving cpu's RUN/STOP state
   component SPI_adc
      generic (
         DELAY      : integer;
         ADDR_WIDTH : integer;
         DATA_WIDTH : integer;
         ADDR_OUT_WIDTH : integer;
         BASE_ADDR  : integer
      );
      port (
         CLK      : in  std_logic;

         CS       : in  std_logic;
         DO       : in  std_logic;
         DO_VLD   : in  std_logic;
         DI       : out std_logic;
         DI_REQ   : in  std_logic;

         ADDR     : out std_logic_vector (ADDR_OUT_WIDTH-1 downto 0);
         DATA_OUT : out std_logic_vector (DATA_WIDTH-1 downto 0);
         DATA_IN  : in  std_logic_vector (DATA_WIDTH-1 downto 0);

         WRITE_EN : out std_logic;
         READ_EN  : out std_logic
      );
   end component;

   -- Picoblaze Processor
   component picocpu
      port (
         -- Common interface
         CLK           : in std_logic;
         RST           : in std_logic;
         -- Core enable
         CORE_EN       : in std_logic;
         -- bus_mux and bean interface
         PORT_ID       : out std_logic_vector(7 downto 0);
         WRITE_STROBE  : out std_logic;
         READ_STROBE   : out std_logic;
         OUT_PORT      : out std_logic_vector(7 downto 0);
         IN_PORT       : in std_logic_vector(7 downto 0);
         -- Interrupt interface
         INTERRUPT     : in std_logic;
         INTERRUPT_ACK : out std_logic
      );
   end component;

   signal cpu_state_reg : std_logic;

   signal state_we      : std_logic;
   signal state_data    : std_logic_vector(7 downto 0);

   signal cpu_write_strobe : std_logic;
   signal cpu_read_strobe  : std_logic;
   signal cpu_port_id      : std_logic_vector(7 downto 0);
   signal cpu_port_out     : std_logic_vector(7 downto 0);
   signal cpu_port_in      : std_logic_vector(7 downto 0);

   signal rst              : std_logic;
   signal clk              : std_logic;
   signal irq              : std_logic;
   signal irq_ack          : std_logic;

-- <constants>











-- </constants>

-- <signals>













-- </signals>

begin

   rst <= P3M(0);
   clk <= SMCLK;

   SPI_CLK <= clk;
   SPI_RST <= rst;

   KIN <= "0111"; -- only first row enabled

   -- SPI run/stop
   spi_cpu_control: SPI_adc
      generic map (
         DELAY      => 1,
         ADDR_WIDTH => 8,
         DATA_WIDTH => 8,
         ADDR_OUT_WIDTH => 1,
         BASE_ADDR  => 16#C0#
      )
      port map (
         CLK      => clk,

         CS       => SPI_CS,
         DO       => SPI_DO,
         DO_VLD   => SPI_DO_VLD,
         DI       => SPI_DI,
         DI_REQ   => SPI_DI_REQ,

         ADDR     => open,
         DATA_OUT => state_data,
         DATA_IN  => "00000000",
         WRITE_EN => state_we,
         READ_EN  => open
      );

   -- PicoCPU
   i_picocpu: picocpu
      port map (
         -- Common interface
         CLK           => clk,
         RST           => rst,
         -- Core enable
         CORE_EN       => cpu_state_reg,
         -- bus_mux and bean interface
         PORT_ID       => cpu_port_id,
         WRITE_STROBE  => cpu_write_strobe,
         READ_STROBE   => cpu_read_strobe,
         OUT_PORT      => cpu_port_out,
         IN_PORT       => cpu_port_in,
         -- Interrupt interface
         INTERRUPT     => irq,
         INTERRUPT_ACK => irq_ack
      );

   run_stop: process(CLK)
   begin
      if CLK'EVENT and CLK = '1' then
         if rst = '1' then
            cpu_state_reg <= '1';
         elsif (state_we = '1') then
            cpu_state_reg <= state_data(0);
         end if;
      end if;
   end process;

-- <bus_mux>





















-- </bus_mux>

-- <beans>




























































-- </beans>

end arch_demo;
