%{                                            /* -*- C++ -*- */
# include <cstdlib>
# include <errno.h>
# include <limits.h>
# include <string.h>
# include "driver.h"
# include "parser.tab.hh"
//# include "global_defs.h"
/* Work around an incompatibility in flex (at least versions
   2.5.31 through 2.5.33): it generates code that does
   not conform to C89.  See Debian bug 333231
   <http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=333231>.  */
# undef yywrap
# define yywrap() 1

/* By default yylex returns int, we use token_type.
   Unfortunately yyterminate by default returns 0, which is
   not of token_type.  */
#define yyterminate() return token::END

typedef yy::vhdl_parser::token token;

// zásobník (v poli) úrovní zanoření a ukazatel (index) do něj
//YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
//int include_stack_ptr = 0;

%}

%option debug

	/* Prozat� jen jeden vstupn�soubor */
%option noyywrap

%x extended_identifier
%x string_literal
%x comment


_LCASE_LETTER_				[abcdefghijklmnopqrstuvwxyzÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÑÒÓÔÕÖØÙÚÛÜ]
_UPCASE_LETTER_				[ABCDEFGHIJKLMNOPQRSTUVWXYZßàáâãäåæçèéêëìíîï∂ñòóôõöøùúûüýṕÿ]
_LETTER_					{_LCASE_LETTER_}|{_UPCASE_LETTER_}
_DIGIT_						[0-9]
_SPECIAL_CHARACTER_			[\]#&'"()*+,-./:;<=>[_|]
_SPACE_CHARACTER_			[ \xA0]
	/* Mezera a NBSP (160 ASCII) */
_OTHER_SPECIAL_CHARACTER_	[!$%@?\^`{}~�]
	/* To poslední je soft hyphen (ASCII 173) */
_FORMAT_EFFECTOR_			[\x09\x0B\x0D\x0A\x0C]
	/* horizontal tabulation, vertical tabulation, carriage return, line feed, and form feed */
_BASIC_GRAPHIC_CHARACTER_	{_UPCASE_LETTER_}|{_DIGIT_}|{_SPECIAL_CHARACTER_}|{_SPACE_CHARACTER_}
_GRAPHIC_CHARACTER_			{_BASIC_GRAPHIC_CHARACTER_}|{_LCASE_LETTER_}|{_OTHER_SPECIAL_CHARACTER_}
_BASIC_CHARACTER_			{_BASIC_GRAPHIC_CHARACTER_}|{_FORMAT_EFFECTOR_}

_BASIC_IDENTIFIER_			{_LETTER_}([_]?({_LETTER_}|{_DIGIT_}))*

_INTEGER_					{_DIGIT_}([_]?{_DIGIT_})*
_EXPONENT_					E([+]|[-])?{_INTEGER_}
_DECIMAL_LITERAL_			{_INTEGER_}([.]{_INTEGER_})?({_EXPONENT_})?

_EXTENDED_DIGIT_			{_DIGIT_}|{_LETTER_}
	/* Pokud bude potřeba zpracovávat hodnotu based_integer, bude lepší ho implementovat pomocí startcondition a v ní udělat jeho převod na něco normálnějšího nebo použít pomocný scanner a v něm to zpracovat... */
_BASED_INTEGER_				{_EXTENDED_DIGIT_}([_]?{_EXTENDED_DIGIT_})*
_BASED_LITERAL_				{_INTEGER_}#{_BASED_INTEGER_}([.]{_BASED_INTEGER_})?#{_EXPONENT_}

_ABSTRACT_LITERAL_			{_DECIMAL_LITERAL_}|{_BASED_LITERAL_}

_CHARACTER_LITERAL_			[']{_GRAPHIC_CHARACTER_}[']



%%

	/* Start-conditions */
\\					BEGIN(extended_identifier);
\"					BEGIN(string_literal);
	/*"--"				BEGIN(comment);*/

	/* Klíčová slova */
abs                 return token::T_ABS;
access              return token::T_ACCESS;
after               return token::T_AFTER;
alias               return token::T_ALIAS;
all                 return token::T_ALL;
and                 return token::T_AND;
architecture        return token::T_ARCHITECTURE;
array               return token::T_ARRAY;
assert              return token::T_ASSERT;
attribute           return token::T_ATTRIBUTE;
begin               return token::T_BEGIN;
block               return token::T_BLOCK;
body                return token::T_BODY;
buffer              return token::T_BUFFER;
bus                 return token::T_BUS;
case                return token::T_CASE;
component           return token::T_COMPONENT;
configuration       return token::T_CONFIGURATION;
constant            return token::T_CONSTANT;
disconnect          return token::T_DISCONNECT;
downto              return token::T_DOWNTO;
else                return token::T_ELSE;
elsif               return token::T_ELSIF;
end                 return token::T_END;
entity              return token::T_ENTITY;
exit                return token::T_EXIT;
file                return token::T_FILE;
for                 return token::T_FOR;
function            return token::T_FUNCTION;
generate            return token::T_GENERATE;
generic             return token::T_GENERIC;
group               return token::T_GROUP;
guarded             return token::T_GUARDED;
if                  return token::T_IF;
impure              return token::T_IMPURE;
in                  return token::T_IN;
inertial            return token::T_INERTIAL;
inout               return token::T_INOUT;
is                  return token::T_IS;
label               return token::T_LABEL;
library             return token::T_LIBRARY;
linkage             return token::T_LINKAGE;
literal             return token::T_LITERAL;
loop                return token::T_LOOP;
map                 return token::T_MAP;
mod                 return token::T_MOD;
nand                return token::T_NAND;
new                 return token::T_NEW;
next                return token::T_NEXT;
nor                 return token::T_NOR;
not                 return token::T_NOT;
null                return token::T_NULL;
of                  return token::T_OF;
on                  return token::T_ON;
open                return token::T_OPEN;
or                  return token::T_OR;
others              return token::T_OTHERS;
out                 return token::T_OUT;
package             return token::T_PACKAGE;
port                return token::T_PORT;
postponed           return token::T_POSTPONED;
procedural          return token::T_PROCEDURAL;		/* nebylo v gramatice */
procedure           return token::T_PROCEDURE;
process             return token::T_PROCESS;
protected           return token::T_PROTECTED;
pure                return token::T_PURE;
range               return token::T_RANGE;
record              return token::T_RECORD;
reference           return token::T_REFERENCE;		/* nebylo v gramatice */
register            return token::T_REGISTER;
reject              return token::T_REJECT;
rem                 return token::T_REM;
report              return token::T_REPORT;
return              return token::T_RETURN;
rol                 return token::T_ROL;
ror                 return token::T_ROR;
select              return token::T_SELECT;
severity            return token::T_SEVERITY;
shared              return token::T_SHARED;
signal              return token::T_SIGNAL;
sla                 return token::T_SLA;
sll                 return token::T_SLL;
sra                 return token::T_SRA;
srl                 return token::T_SRL;
subtype             return token::T_SUBTYPE;
then                return token::T_THEN;
to                  return token::T_TO;
transport           return token::T_TRANSPORT;
type                return token::T_TYPE;
unaffected          return token::T_UNAFFECTED;
units               return token::T_UNITS;
until               return token::T_UNTIL;
use                 return token::T_USE;
variable            return token::T_VARIABLE;
wait                return token::T_WAIT;
when                return token::T_WHEN;
while               return token::T_WHILE;
with                return token::T_WITH;
xnor                return token::T_XNOR;
xor                 return token::T_XOR;

	/* operátory */

\+                  return token::T_PLUS;
-                   return token::T_MINUS;
&                   return token::T_AMPERSAND;
,                   return token::T_COMA;
:                   return token::T_COLON;
;                   return token::T_SEMICOLON;
\'                  return token::T_APOSTROPHE;
\.                  return token::T_DOT;
\|                  return token::T_PIPE;
\/                  return token::T_SLASH;
\(                  return token::T_LPAREN;
\)                  return token::T_RPAREN;
=                   return token::T_EQUAL;
\<                  return token::T_LESS;
>                   return token::T_BIGGER;
\[                  return token::T_LBRACE;
]                   return token::T_RBRACE;
\*                   return token::T_STAR;
\<=					return token::T_LESS_THAN_OR_EQUAL;
>=					return token::T_GREATER_THAN_OR_EQUAL;
\/=					return token::T_INEQUALITY;
\*\*				return token::T_DOUBLESTAR;
:=					return token::T_VARIABLE_ASSIGNMENT;
=>					return token::T_ARROW;
\<>					return token::T_BOX;


	/* Identifikátory a spol */
{_BASIC_IDENTIFIER_}		{yylval->sval = new std::string(yytext); return token::T_BASIC_IDENTIFIER;}
{_ABSTRACT_LITERAL_}		return token::T_ABSTRACT_LITERAL;
{_CHARACTER_LITERAL_}		return token::T_CHARACTER_LITERAL;


	/* Pokud bude potřeba zpracováat hodnotu bit_string_literal, bude lepšá ho implementovat pomocí startcondition a v ní udělat jeho převod na něco normálnějšího nebo použít pomocný scanner a v něm to zpracovat... */
"B\"({EXTENDED_DIGIT}([_]?{EXTENDED_DIGIT})*)?\""	return token::T_BIT_STRING_LITERAL_B;
"O\"({EXTENDED_DIGIT}([_]?{EXTENDED_DIGIT})*)?\""	return token::T_BIT_STRING_LITERAL_O;
"X\"({EXTENDED_DIGIT}([_]?{EXTENDED_DIGIT})*)?\""	return token::T_BIT_STRING_LITERAL_X;


--.*\n		yylineno++;			/* Komentář */

	/* A ten zbytek... */
\n			yylineno++;	/* Další řádek */
{_FORMAT_EFFECTOR_}|\ 		// mezery
			/* cokoliv, co se nechytne... */
.			printf("Lexical error on line %d. Invalid character %c.", yylineno, yytext[0]);



	/* Start-condition rules */

<extended_identifier>{
	\n						{yylineno++;}
	{_GRAPHIC_CHARACTER_}		; /* Přijmi všechno, co se dá. Kvůli rychlosti by bylo lepší to udělat jako řetězec, al epřijalo by to i konce řádků a nedaly by se počítat. Standard natvrdo říká, že mezi format effectory patří i LF a CR, zatímco flex konec řádku bere vždy v závislosti na konkrétním OS. */

	"\\\\"					; /* ošetřit backslashítko */
	\\						{
								BEGIN(INITIAL);	/* Konec identifikátoru */
								yylval->sval = new std::string(yytext);
								return token::T_EXTENDED_IDENTIFIER;
							}
}

<string_literal>{
	\n						{yylineno++;}
	[^"\n]		; /* Přijmi všechno, co se dá. Kvůli rychlosti by bylo lepší to udělat jako řetězec, al epřijalo by to i konce řádků a nedaly by se počítat. Standard natvrdo říká, že mezi format effectory patří i LF a CR, zatímco flex konec řádku bere vždy v závislosti na konkrétním OS. */
	"\"\""					; /* ošetřit uvozovky */
	"\""						{
								BEGIN(INITIAL);	/* Konec řetězce */
								yylval->sval = new std::string(yytext);
								return token::T_STRING_LITERAL;
							}
}


%%

void vhdl_driver::scan_begin()
{
	if (!(yyin = fopen (file.c_str(), "r")))
		error (std::string ("Cannot open ") + file);
	// TODO - vyvolat vyjimku pro osetreni neexistujiciho souboru
}


void vhdl_driver::scan_end()
{
	fclose (yyin);
}
