%skeleton "lalr1.cc"                          /*  -*- C++ -*- */
%require "2.1a"
%defines
%define "parser_class_name" "vhdl_parser"
%{
#include <string>
#include <set>
class vhdl_driver;
%}
// The parsing context.
%parse-param { vhdl_driver& driver }
%lex-param   { vhdl_driver& driver }
%locations
%initial-action
{
  // Initialize the initial location.
  @$.begin.filename = @$.end.filename = &driver.file;
};
%debug
%error-verbose

%{
	enum entity_header_enum_type {eh_generic, eh_port, eh_generic_port, eh_none};
	typedef std::set<std::string> _t_id_set;
%}

// Symbols.
%union
{
	int ival;
	std::string *sval;
	entity_header_enum_type eh_val;
	_t_id_set * id_set_val;
};

%{
#include "driver.h"
#include "symtable.h"
static std::string * p_empty_str = new std::string;
%}

//TODO - Při zpracování ID listů se vytváří spousta množin identfikátorů, které, podud nejsou potřeba, se neuvolňují z paměti. Pro testování to příliš nevadí, pokud je dostatek paměti. Pro serióznější použití se to však musí ošetřit.



%token	T_ACCESS
%token	T_AFTER
%token	T_ALIAS
%token	T_ALL
%token	T_ARCHITECTURE
%token	T_ARRAY
%token	T_ASSERT
%token	T_ATTRIBUTE
%token	T_BEGIN
%token	T_BLOCK
%token	T_BODY
%token	T_BUFFER
%token	T_BUS
%token	T_CASE
%token	T_COMPONENT
%token	T_CONFIGURATION
%token	T_CONSTANT
%token	T_DISCONNECT
%token	T_DOWNTO
%token	T_ELSE
%token	T_ELSIF
%token	T_END
%token	T_ENTITY
%token	T_EXIT
%token	T_FILE
%token	T_FOR
%token	T_FUNCTION
%token	T_GENERATE
%token	T_GENERIC
%token	T_GROUP
%token	T_GUARDED
%token	T_IF
%token	T_IMPURE
%token	T_IN
%token	T_INERTIAL
%token	T_INOUT
%token	T_IS
%token	T_LABEL
%token	T_LIBRARY
%token	T_LINKAGE
%token	T_LITERAL
%token	T_LOOP
%token	T_MAP
%token	T_NEW
%token	T_NEXT
%token	T_NULL
%token	T_OF
%token	T_ON
%token	T_OPEN
%token	T_OTHERS
%token	T_OUT
%token	T_PACKAGE
%token	T_PORT
%token	T_POSTPONED
%token	T_PROCEDURAL
%token	T_PROCEDURE
%token	T_PROCESS
%token	T_PROTECTED
%token	T_PURE
%token	T_RANGE
%token	T_RECORD
%token	T_REFERENCE
%token	T_REGISTER
%token	T_REJECT
%token	T_REPORT
%token	T_RETURN
%token	T_SELECT
%token	T_SEVERITY
%token	T_SHARED
%token	T_SIGNAL
%token	T_SUBTYPE
%token	T_THEN
%token	T_TO
%token	T_TRANSPORT
%token	T_TYPE
%token	T_UNAFFECTED
%token	T_UNITS
%token	T_UNTIL
%token	T_USE
%token	T_VARIABLE
%token	T_WAIT
%token	T_WHEN
%token	T_WHILE
%token	T_WITH
//%token	T_DOUBLESTAR
%token	T_VARIABLE_ASSIGNMENT
%token	T_ARROW
%token	T_BOX
%token	<sval>	T_BASIC_IDENTIFIER
%token	T_CHARACTER_LITERAL
%token	T_ABSTRACT_LITERAL
%token	T_BIT_STRING_LITERAL_B
%token	T_BIT_STRING_LITERAL_O
%token	T_BIT_STRING_LITERAL_X
%token	<sval>	T_EXTENDED_IDENTIFIER
%token	<sval>	T_STRING_LITERAL

//%token T_PLUS
//%token T_MINUS
//%token T_AMPERSAND
%token T_COMA
%token T_COLON
%token T_SEMICOLON
%token T_APOSTROPHE
%token T_DOT
%token T_PIPE
//%token T_SLASH
%token T_LPAREN
%token T_RPAREN
//%token T_EQUAL
//%token T_LESS
//%token T_BIGGER
%token T_LBRACE
%token T_RBRACE
//%token T_STAR


%token T__PORT_
%token T__SIGNAL_
%token T__FUNCTION_
%token T__TYPE_
%token T__PARAMETER_
%token T__VARIABLE_
%token T__GENERIC_
%token T__SUBTYPE_
%token T__FILE_
%token T__UNIT_
%token T__RANGE_
%token T__ELEMENT_
%token T__ARCHITECTURE_
%token T__GENERATE_STATEMENT_
// Zakomentované terminály jsou uvedeny později v sekci priorit jako pokus o řešení S/R konfliktů, ale v současné verzi bisona je to k ničemu...

%token END



//%token T_BOGUS


%type	<ival>			architecture_body
%type	<ival>			primary_unit_declaration
%type	<ival>			secondary_unit_declaration
%type	<ival>			generate_statement
%type	<ival>			subprogram_specification
%type	<id_set_val>	identifier_list
%type	<id_set_val>	interface_list
%type	<id_set_val>	interface_declaration
%type	<id_set_val>	interface_element
%type	<id_set_val>	signal_if_decl
%type	<id_set_val>	variable_if_decl
%type	<id_set_val>	constant_if_decl
%type	<sval>			identifier
%type	<sval>			label
%type	<sval>			label_wrapped
%type	<sval>			designator
%type	<sval>			operator_symbol
//%type	<sval>			prefix
//%type	<sval>			suffix
%type	<sval>			name_wr
%type	<sval>			simple_name
%type	<sval>			simple_name_wrapped
%type	<sval>			attribute_name
%type	<sval>			attribute_name_wr
%type	<sval>			attribute_name_wrapped
%type	<sval>			attribute_designator
%type	<eh_val>		entity_header


%left T_AND T_OR T_NAND T_NOR T_XOR T_XNOR
%left T_EQUAL T_INEQUALITY T_LESS T_LESS_THAN_OR_EQUAL T_BIGGER T_GREATER_THAN_OR_EQUAL
%left T_SLL T_SRL T_SLA T_SRA T_ROL T_ROR
%left T_PLUS T_MINUS T_AMPERSAND
%left T_UNARY_PLUS T_UNARY_MINUS
%left T_STAR T_SLASH T_MOD T_REM
%left T_DOUBLESTAR T_ABS T_NOT


//Pomocný terminál pro nastavení precedence pravidla
%left T_AUX_REDUCE_MODIFIER



%%

start
	: design_file
	;



/*#####################################################
  #        Design entities and conﬁgurations          #
  #####################################################*/


/********************************** Entity declarations **********************************/

entity_declaration
	: T_ENTITY identifier T_IS
	  entity_header
		{switch ($4) {
			case eh_generic:
				driver.p_symtable->addToSet(std::symtable::s_generic, *$2);
				break;
			case eh_port:
				driver.p_symtable->addToSet(std::symtable::s_port, *$2);
				break;
			case eh_generic_port:
				driver.p_symtable->addToSet(std::symtable::s_generic, *$2);
				driver.p_symtable->addToSet(std::symtable::s_port, *$2);
				break;
			default:
				break;
			}
		}
	  entity_declarative_part
	  entity_declaration_begin_entity_statement_part_optional
	  T_END entity_optional simple_name_optional T_SEMICOLON
	;

entity_declaration_begin_entity_statement_part_optional
	: //empty
	| T_BEGIN entity_statement_part
	;


/********** Entity header **********/

entity_header
	: //empty
		{$$ = eh_none;}
	| generic_clause
		{$$ = eh_generic;}
	| port_clause
		{$$ = eh_port;}
	| generic_clause port_clause
		{$$ = eh_generic_port;}
	;

generic_clause
	: T_GENERIC T_LPAREN generic_list T_RPAREN T_SEMICOLON
	;

port_clause
	: T_PORT T_LPAREN port_list T_RPAREN T_SEMICOLON
	;


/********** Generics **********/

generic_list
	: interface_list
	;


/********** Ports **********/

port_list
	: interface_list
	;


/********** Entity declarative part **********/

entity_declarative_part
	: //empty
	| entity_declarative_part entity_declarative_item
	;

entity_declarative_item
	: subprogram_declaration
	| subprogram_body
	| type_declaration
	| subtype_declaration
	| constant_declaration
	| signal_declaration
	| variable_declaration
	| file_declaration
	| alias_declaration
	| attribute_declaration
	| attribute_specification
	| disconnection_specification
	| use_clause
	| group_template_declaration
	| group_declaration
	;


/********** Entity statement part **********/

entity_statement_part
	: //empty
	| entity_statement_part entity_statement
	;

entity_statement
	: concurrent_assertion_statement
	| concurrent_procedure_call_statement	// doplněno _statement, protože concurrent_procedure_call neexistuje - zkontrolovat, případně dořešit
	| process_statement
	;



/********************************** Architecture bodies **********************************/

architecture_body
	: T_ARCHITECTURE identifier {driver.p_symtable->addToSet(std::symtable::s_architecture, *$2);} T_OF name T_IS
		architecture_declarative_part
		T_BEGIN
			architecture_statement_part
		T_END architecture_optional simple_name_optional T_SEMICOLON
			{$$ = 0;}	// hloupá akce, aby bison nebrblal, že mu nesedí typy...
	;


/********** Architecture declarative part **********/

architecture_declarative_part
	: //empty
	| architecture_declarative_part block_declarative_item
	;

block_declarative_item
	: subprogram_declaration
	| subprogram_body
	| type_declaration
	| subtype_declaration
	| constant_declaration
	| signal_declaration
	| variable_declaration
	| file_declaration
	| alias_declaration
	| component_declaration
	| attribute_declaration
	| attribute_specification
	| configuration_specification
	| disconnection_specification
	| use_clause
	| group_template_declaration
	| group_declaration
	;


/********** Architecture statement part **********/

architecture_statement_part
	: //empty
	| architecture_statement_part concurrent_statement
	;


/********************************** Conﬁguration declarations **********************************/

configuration_declaration
	: T_CONFIGURATION identifier T_OF name T_IS
	  configuration_declarative_part
	  block_configuration
	  T_END configuration_optional simple_name_optional T_SEMICOLON
	;

configuration_declarative_part
	: //empty
	| configuration_declarative_part configuration_declarative_item
	;

configuration_declarative_item
	: use_clause
	| attribute_specification
	| group_declaration
	;


/********** Block configuration **********/

block_configuration
	: T_FOR
		block_specification block_configuration_use_clause_iterate block_configuration_configuration_item_iterate
	  T_END T_FOR T_SEMICOLON
	;

block_configuration_use_clause_iterate
	: //empty
	| block_configuration_use_clause_iterate use_clause
	;

block_configuration_configuration_item_iterate
	: //empty
	| block_configuration_configuration_item_iterate configuration_item
	;

block_specification
	: name T__ARCHITECTURE_
	| label
	| label T__GENERATE_STATEMENT_ T_LPAREN index_specification T_RPAREN
	;

index_specification
	: discrete_range
	| expression
	;

configuration_item
	: block_configuration
	| component_configuration
	;


/********** Component configuration **********/

component_configuration
	: T_FOR component_specification
	  component_configuration_binding_indication_optional
	  component_configuration_block_configuration_optional
	  T_END T_FOR T_SEMICOLON
	;

component_configuration_binding_indication_optional
 : // empty
	| binding_indication T_SEMICOLON
	;

component_configuration_block_configuration_optional
 : // empty
	| block_configuration
	;



/*#####################################################
  #              Subprograms and packages             #
  #####################################################*/


/********************************** Subprogram declarations **********************************/

subprogram_declaration
	: subprogram_specification T_SEMICOLON
	;

subprogram_specification
	: T_PROCEDURE designator
		{$$ = 0;}
	| T_PROCEDURE designator T_LPAREN formal_parameter_list T_RPAREN
		{$$ = 0;}
	| subprogram_specification_pure_impure_optional T_FUNCTION designator {driver.p_symtable->addToSet(std::symtable::s_function, *$3);} T_RETURN type_mark {$$ = 0;}
	| subprogram_specification_pure_impure_optional T_FUNCTION designator {driver.p_symtable->addToSet(std::symtable::s_function, *$3);} T_LPAREN formal_parameter_list T_RPAREN T_RETURN type_mark {$$ = 0;}
	;

subprogram_specification_pure_impure_optional
	: //empty
	| T_PURE
	| T_IMPURE
	;

designator
	: identifier
		{$$ = $1;}
	| operator_symbol
		{$$ = $1;}
	;

operator_symbol
	: T_STRING_LITERAL
		{$$ = $1;}
	;


/********** Formal parameters **********/

formal_parameter_list
	: interface_list
		{
			driver.p_symtable->addSetToSet(std::symtable::s_parameter, $1);
			delete $1;
		}
	;


/********** Subprogram bodies **********/

subprogram_body
	: subprogram_specification T_IS
	  subprogram_declarative_part
	  T_BEGIN
	  subprogram_statement_part
	  T_END subprogram_body_subprogram_optional T_SEMICOLON
	;

subprogram_body_subprogram_optional
	: //empty
	| subprogram_kind
	| designator
	| subprogram_kind designator
	;

subprogram_declarative_part
	: //empty
	| subprogram_declarative_part subprogram_declarative_item
	;

subprogram_declarative_item
	: subprogram_declaration
	| subprogram_body
	| type_declaration
	| subtype_declaration
	| constant_declaration
	| variable_declaration
	| file_declaration
	| alias_declaration
	| attribute_declaration
	| attribute_specification
	| use_clause
	| group_template_declaration
	| group_declaration
	;

subprogram_statement_part
	: //empty
	| subprogram_statement_part sequential_statement
	;

subprogram_kind
	: T_PROCEDURE
	| T_FUNCTION
	;


/********************************** Subprogram overloading **********************************/

/********** Signatures **********/

signature
	: T_LBRACE T_RBRACE
	| T_LBRACE signature_type_mark_iterate T_RBRACE
	| T_LBRACE T_RETURN type_mark T_RBRACE
	| T_LBRACE signature_type_mark_iterate T_RETURN type_mark T_RBRACE
	;

signature_type_mark_iterate
	: type_mark
	| signature_type_mark_iterate T_COMA type_mark
	;


/********************************** Package declarations **********************************/

package_declaration
	: T_PACKAGE identifier T_IS
	  package_declarative_part
	  T_END package_optional simple_name_optional T_SEMICOLON
	;

package_declarative_part
	: //empty
	| package_declarative_part package_declarative_item
	;

package_declarative_item
	: subprogram_declaration
	| type_declaration
	| subtype_declaration
	| constant_declaration
	| signal_declaration
	| variable_declaration
	| file_declaration
	| alias_declaration
	| component_declaration
	| attribute_declaration
	| attribute_specification
	| disconnection_specification
	| use_clause
	| group_template_declaration
	| group_declaration
	;


/********************************** Package bodies **********************************/

package_body
	: T_PACKAGE T_BODY simple_name T_IS
	  package_body_declarative_part
	  T_END package_body_package_body_optional simple_name_optional T_SEMICOLON
	;

package_body_package_body_optional
	: //empty
	| T_PACKAGE T_BODY
	;

package_body_declarative_part
	: //empty
	| package_body_declarative_part package_body_declarative_item
	;

package_body_declarative_item
	: subprogram_declaration
	| subprogram_body
	| type_declaration
	| subtype_declaration
	| constant_declaration
	| variable_declaration
	| file_declaration
	| alias_declaration
	| use_clause
	| group_template_declaration
	| group_declaration
	;



/*#####################################################
  #                       Types                       #
  #####################################################*/


/********************************** Scalar types **********************************/

scalar_type_definition
	: enumeration_type_definition
	| range_constraint
	| physical_type_definition
	;

range_constraint
	: T_RANGE range
	;

range
	: attribute_name T__RANGE_
	| simple_expression direction simple_expression
	;

direction
	: T_TO
	| T_DOWNTO
	;



/********** Enumeration types **********/

enumeration_type_definition
	: T_LPAREN enumeration_type_definition_iterate T_RPAREN
	;

enumeration_type_definition_iterate
	: enumeration_literal
	| enumeration_type_definition_iterate T_COMA enumeration_literal
	;

enumeration_literal
	: identifier
	| T_CHARACTER_LITERAL
	;


/********** Physical types **********/

physical_type_definition
	: range_constraint
	  T_UNITS
		primary_unit_declaration
		physical_type_definition_secondary_unit_declaration_iterate
	  T_END T_UNITS simple_name_optional
	;

primary_unit_declaration
	: identifier {driver.p_symtable->addToSet(std::symtable::s_unit, *$1);} T_SEMICOLON
			{$$ = 0;}	// hloupá akce, aby bison nebrblal, že mu nesedí typy...
	;

physical_type_definition_secondary_unit_declaration_iterate
	: //empty
	| physical_type_definition_secondary_unit_declaration_iterate secondary_unit_declaration
	;

secondary_unit_declaration
	: identifier {driver.p_symtable->addToSet(std::symtable::s_unit, *$1);} T_EQUAL physical_literal T_SEMICOLON
			{$$ = 0;}	// hloupá akce, aby bison nebrblal, že mu nesedí typy...
	;

physical_literal
	: name T__UNIT_
	| T_ABSTRACT_LITERAL name T__UNIT_
	;


/********************************** Composite types **********************************/

composite_type_definition
	: array_type_definition
	| record_type_definition
	;


/********** Array types **********/

array_type_definition
	: unconstrained_array_definition
	| constrained_array_definition
	;

unconstrained_array_definition
	: T_ARRAY T_LPAREN unconstrained_array_definition_index_subtype_definition_iterate T_RPAREN
	  T_OF subtype_indication
	;

unconstrained_array_definition_index_subtype_definition_iterate
	: index_subtype_definition
	| unconstrained_array_definition_index_subtype_definition_iterate T_COMA index_subtype_definition
	;

constrained_array_definition
	: T_ARRAY index_constraint T_OF subtype_indication
	;

index_subtype_definition
	: type_mark T_RANGE T_BOX
	;

index_constraint
	: T_LPAREN index_constraint_discrete_range_iterate T_RPAREN
	;

index_constraint_discrete_range_iterate
	: discrete_range
	| index_constraint_discrete_range_iterate T_COMA discrete_range
	;

discrete_range
	: subtype_indication
	| range
	;


/********** Record types **********/

record_type_definition
	: T_RECORD
	    record_type_definition_element_declaration_iterate
	  T_END T_RECORD simple_name_optional
	;

record_type_definition_element_declaration_iterate
	: element_declaration
	| record_type_definition_element_declaration_iterate element_declaration
	;

element_declaration
	: identifier_list T_COLON element_subtype_definition T_SEMICOLON {driver.p_symtable->addSetToSet(std::symtable::s_element, $1);}
	;

identifier_list
	: identifier
		{
			_t_id_set * p_set = new(_t_id_set);
			p_set->insert(*$1);
			$$ = p_set;
		}
	| identifier_list T_COMA identifier
		{
			$1->insert(*$3);
		}
	;

element_subtype_definition
	: subtype_indication
	;



/********************************** Access types **********************************/

access_type_definition
	: T_ACCESS subtype_indication
	;


/********** Incomplete type declarations **********/

incomplete_type_declaration
	: T_TYPE identifier {driver.p_symtable->addToSet(std::symtable::s_type, *$2);} T_SEMICOLON
	;



/********************************** File types **********************************/

file_type_definition
	: T_FILE T_OF type_mark
	;



/********************************** Protected types **********************************/

protected_type_definition
	: protected_type_declaration
	| protected_type_body
	;


/********** Protected type declarations **********/

protected_type_declaration
	: T_PROTECTED
	    protected_type_declarative_part
	  T_END T_PROTECTED simple_name_optional
	;

protected_type_declarative_part
	: //empty
	| protected_type_declarative_part protected_type_declarative_item
	;

protected_type_declarative_item
	: subprogram_declaration		// tak tady se liší zápis na straně 50 standardu od strany 231. Co je správně?
	| attribute_specification
	| use_clause
	;


/********** Protected type bodies **********/

protected_type_body
	: T_PROTECTED T_BODY
	    protected_type_body_declarative_part
	  T_END T_PROTECTED T_BODY simple_name_optional	// Doplněno podtržítko, simple ani name samostatně neexistují :-(
	;

protected_type_body_declarative_part
	: //empty
	| protected_type_body_declarative_part protected_type_body_declarative_item
	;

protected_type_body_declarative_item
	: subprogram_declaration
	| subprogram_body
	| type_declaration
	| subtype_declaration
	| constant_declaration
	| variable_declaration
	| file_declaration
	| alias_declaration
	| attribute_declaration
	| attribute_specification
	| use_clause
	| group_template_declaration
	| group_declaration
	;





/*#####################################################
  #                   Declarations                    #
  #####################################################*/


/*declaration
	: type_declaration
	| subtype_declaration
	| object_declaration
	| interface_declaration
	| alias_declaration
	| attribute_declaration
	| component_declaration
	| group_template_declaration
	| group_declaration
	| entity_declaration
	| configuration_declaration
	| subprogram_declaration
	| package_declaration
	| primary_unit
	| architecture_body
	;*/


/********************************** Type declarations **********************************/

type_declaration
	: full_type_declaration
	| incomplete_type_declaration
	;

full_type_declaration
	: T_TYPE identifier {driver.p_symtable->addToSet(std::symtable::s_type, *$2);} T_IS type_definition T_SEMICOLON
	;

type_definition
	: scalar_type_definition
	| composite_type_definition
	| access_type_definition
	| file_type_definition
	| protected_type_definition
	;


/********************************** Subtype declarations **********************************/

subtype_declaration
	: T_SUBTYPE identifier {driver.p_symtable->addToSet(std::symtable::s_subtype, *$2);} T_IS subtype_indication T_SEMICOLON
	;

subtype_indication
	: type_mark
	| name type_mark
	| type_mark constraint
	| name type_mark constraint
	;

type_mark
	: name T__TYPE_
	| name T__SUBTYPE_
	;

constraint
	: range_constraint
	| index_constraint
	;


/********************************** Objects **********************************/


/********** Object declarations **********/

/*object_declaration
	: constant_declaration
	| signal_declaration
	| variable_declaration
	| file_declaration
	;*/

constant_declaration
	: T_CONSTANT identifier_list T_COLON subtype_indication T_SEMICOLON
	| T_CONSTANT identifier_list T_COLON subtype_indication T_VARIABLE_ASSIGNMENT expression T_SEMICOLON
	;

signal_declaration
	: T_SIGNAL identifier_list T_COLON subtype_indication T_SEMICOLON
	| T_SIGNAL identifier_list T_COLON subtype_indication signal_kind T_SEMICOLON
	| T_SIGNAL identifier_list T_COLON subtype_indication T_VARIABLE_ASSIGNMENT expression T_SEMICOLON
	| T_SIGNAL identifier_list T_COLON subtype_indication signal_kind T_VARIABLE_ASSIGNMENT expression T_SEMICOLON
	;

signal_kind
	: T_REGISTER
	| T_BUS
	;

variable_declaration
	: T_VARIABLE identifier_list T_COLON subtype_indication T_SEMICOLON
	| T_SHARED T_VARIABLE identifier_list T_COLON subtype_indication T_SEMICOLON
	| T_VARIABLE identifier_list T_COLON subtype_indication T_VARIABLE_ASSIGNMENT expression T_SEMICOLON
	| T_SHARED T_VARIABLE identifier_list T_COLON subtype_indication T_VARIABLE_ASSIGNMENT expression T_SEMICOLON
	;

file_declaration
	: T_FILE identifier_list T_COLON subtype_indication T_SEMICOLON
		{
			driver.p_symtable->addSetToSet(std::symtable::s_file, $2);
			delete $2;
		}
	| T_FILE identifier_list T_COLON subtype_indication file_open_information T_SEMICOLON
		{
			driver.p_symtable->addSetToSet(std::symtable::s_file, $2);
			delete $2;
		}
	;

file_open_information
	: T_IS file_logical_name
	| T_OPEN expression T_IS file_logical_name
	;

file_logical_name
	: expression
	;


/********** Interface declarations **********/

interface_declaration
	: T_CONSTANT constant_if_decl {$$ = $2;}
	| T_SIGNAL signal_if_decl {driver.p_symtable->addSetToSet(std::symtable::s_signal, $2); $$ = $2;}
	| T_VARIABLE variable_if_decl {driver.p_symtable->addSetToSet(std::symtable::s_variable, $2); $$ = $2;}
	| identifier_list T_COLON T_IN subtype_indication if_decl_var_assign_optional {$$ = $1;}
	| identifier_list T_COLON mode subtype_indication if_decl_next {$$ = $1;}
	| identifier_list T_COLON subtype_indication {$$ = $1;}
	;

constant_if_decl
	: identifier_list T_COLON subtype_indication if_decl_var_assign_optional {$$ = $1;}
	| identifier_list T_COLON T_IN subtype_indication if_decl_var_assign_optional {$$ = $1;}
	;

signal_if_decl
	: identifier_list T_COLON subtype_indication if_decl_var_assign_optional {$$ = $1}
	| identifier_list T_COLON mode subtype_indication if_decl_var_assign_optional {$$ = $1}
	| identifier_list T_COLON subtype_indication T_BUS if_decl_var_assign_optional {$$ = $1}
	| identifier_list T_COLON mode subtype_indication T_BUS if_decl_var_assign_optional {$$ = $1}
	;

variable_if_decl
	: identifier_list T_COLON subtype_indication if_decl_var_assign_optional {$$ = $1}
	| identifier_list T_COLON mode subtype_indication if_decl_var_assign_optional {$$ = $1}
	;

if_decl_next
	: if_decl_var_assign_optional
	| T_BUS if_decl_var_assign_optional
	;

if_decl_var_assign_optional
	: //empty
	| T_VARIABLE_ASSIGNMENT expression
	;




//@@@@@@@@@@@@@@@@ -- řešení R/R konfliktu - viz pravidla pro interface_declaration výše, původní je zde zakomentováno
/*interface_declaration
	: interface_constant_declaration
	| interface_signal_declaration
	| interface_variable_declaration
	| interface_file_declaration
	;

interface_constant_declaration
	: interface_constant_declaration_constant_optional identifier_list T_COLON
	  interface_constant_declaration_in_optional subtype_indication
	  interface_constant_declaration_colon_equal_expression_optional
	;

interface_constant_declaration_constant_optional
	: //empty
	| T_CONSTANT
	;

interface_constant_declaration_in_optional
	: //empty
	| T_IN
	;

interface_constant_declaration_colon_equal_expression_optional
	: //empty
	| T_VARIABLE_ASSIGNMENT expression
	;

interface_signal_declaration
	: interface_signal_declaration_signal_optional identifier_list T_COLON
	  interface_signal_declaration_mode_optional
	  subtype_indication interface_signal_declaration_bus_optional
	  interface_signal_declaration_colon_equal_expression_optional
	;

interface_signal_declaration_signal_optional
	: //empty
	| T_SIGNAL
	;

interface_signal_declaration_mode_optional
	: //empty
	| mode
	;

interface_signal_declaration_bus_optional
	: //empty
	| T_BUS
	;

interface_signal_declaration_colon_equal_expression_optional
	: //empty
	| T_VARIABLE_ASSIGNMENT expression
	;

interface_variable_declaration
	: interface_variable_declaration_variable_optional identifier_list T_COLON
	  interface_variable_declaration_mode_optional
	  subtype_indication interface_variable_declaration_colon_equal_expression_optional
	;

interface_variable_declaration_variable_optional
	: //empty
	| T_VARIABLE
	;

interface_variable_declaration_mode_optional
	: //empty
	| mode
	;

interface_variable_declaration_colon_equal_expression_optional
	: //empty
	| T_VARIABLE_ASSIGNMENT expression
	;*/
//@@@@@@@@@@@@@@@@

/*interface_file_declaration
	: T_FILE identifier_list T_COLON subtype_indication
	;*/

mode
	: T_IN
	| T_OUT
	| T_INOUT
	| T_BUFFER
	| T_LINKAGE
	;

interface_list
	: interface_element {$$ = $1;}
	| interface_list T_SEMICOLON interface_element
		{
			$1->insert($3->begin(), $3->end());		// druhá množina se připojí k první a pak se zničí
			delete $3;
			$$ = $1;
		}
	;

interface_element
	: interface_declaration {$$ = $1;}
	;

association_list
	: association_element
	| association_list T_COMA association_element
	;

// Tady je celkem hnusný konflikt. Nepomáhá ani znalost sémantiky. Pokud je totiž nějaké jméno například jménem signálu, může být zároveň i jménem parametru, protože ten signál je parametrem funkce. Jediné, podle čeho se dá rozlišit zda se má první načtené jméno redukovat na formal_part a nebo actual_part je existence T_ARROW někde později, ale ne později než před první T_RPAREN. pokud to tedy bude potřeba vyřešit, bude nutné použít buď GLR (ale to může zase zpozdit sémantické akce - sekce 1.5.3, strana 19 manuálu bisona z 30. 5. 2006) a nebo na vlastní pěst prohledat vstup a podle toho určit, kterou větev vybrat a prohledané tokeny vrátit zpět do vstupu. Prozatím se dává přednost volbě cesty přes druhé pravidlo, tedy, pokud to jde, vybírá se formal_part.

association_element
	: actual_part
	| formal_part T_ARROW actual_part
	;

formal_part
	: formal_designator
	| name T__FUNCTION_ T_LPAREN formal_designator T_RPAREN
	| type_mark T_LPAREN formal_designator T_RPAREN
	;

formal_designator
	: name T__GENERIC_
	| name T__PORT_
	| name T__PARAMETER_
	;

actual_part
	: actual_designator
	| name T__FUNCTION_ T_LPAREN actual_designator T_RPAREN
	| type_mark T_LPAREN actual_designator T_RPAREN
	;

actual_designator
	: expression
	| name T__SIGNAL_
	| name T__VARIABLE_
	| name T__FILE_
	| T_OPEN
	;



/********************************** Alias declarations **********************************/

alias_declaration
	: T_ALIAS alias_designator alias_declaration_subtype_indication_optional T_IS name
	  alias_declaration_signature_optional T_SEMICOLON
	;

alias_declaration_subtype_indication_optional
	: //empty
	| T_COLON subtype_indication
	;

alias_declaration_signature_optional
	: //empty
	| signature
	;

alias_designator
	: identifier
	| T_CHARACTER_LITERAL
	| operator_symbol
	;



/********************************** Attribute declarations **********************************/

attribute_declaration
	: T_ATTRIBUTE identifier T_COLON type_mark T_SEMICOLON
	;



/********************************** Component declarations **********************************/

component_declaration
	: T_COMPONENT identifier is_optional
	  component_declaration_generic_clause_optional
	  component_declaration_port_clause_optional
	  T_END T_COMPONENT simple_name_optional T_SEMICOLON
	;

component_declaration_generic_clause_optional
	: //empty
	| generic_clause
	;

component_declaration_port_clause_optional
	: //empty
	| port_clause
	;



/********************************** Group template declarations **********************************/

group_template_declaration
	: T_GROUP identifier T_IS T_LPAREN entity_class_entry_list T_RPAREN T_SEMICOLON
	;

entity_class_entry_list
	: entity_class_entry
	| entity_class_entry_list T_COMA entity_class_entry
	;

entity_class_entry
	: entity_class T_BOX
	| entity_class
	;



/********************************** Group declarations **********************************/

group_declaration
	: T_GROUP identifier T_COLON name T_LPAREN group_constituent_list T_RPAREN T_SEMICOLON
	;

group_constituent_list
	: group_constituent
	| group_constituent_list T_COMA group_constituent
	;

group_constituent
	: name
	| T_CHARACTER_LITERAL
	;





/*#####################################################
  #                  Specifications                   #
  #####################################################*/


/********************************** Attribute specification **********************************/

attribute_specification
	: T_ATTRIBUTE attribute_designator T_OF entity_specification T_IS expression T_SEMICOLON
	;

entity_specification
	: entity_name_list T_COLON entity_class
	;

entity_class
	: T_ENTITY
	| T_ARCHITECTURE
	| T_CONFIGURATION
	| T_PROCEDURE
	| T_FUNCTION
	| T_PACKAGE
	| T_TYPE
	| T_SUBTYPE
	| T_CONSTANT
	| T_SIGNAL
	| T_VARIABLE
	| T_COMPONENT
	| T_LABEL
	| T_LITERAL
	| T_UNITS
	| T_GROUP
	| T_FILE
	;

entity_name_list
	: entity_name_list_entity_designator_iterate
	| T_OTHERS
	| T_ALL
	;

entity_name_list_entity_designator_iterate
	: entity_designator
	| entity_name_list_entity_designator_iterate T_COMA entity_designator
	;

entity_designator
	: entity_tag
	| entity_tag signature
	;

entity_tag
	: simple_name
	| T_CHARACTER_LITERAL
	| operator_symbol
	;



/********************************** Configuration specification **********************************/

configuration_specification
	: T_FOR component_specification binding_indication T_SEMICOLON
	;

component_specification
	: instantiation_list T_COLON name
	;

instantiation_list
	: instantiation_list_instantiation_label_iterate
	| T_OTHERS
	| T_ALL
	;

instantiation_list_instantiation_label_iterate
	: label
	| instantiation_list_instantiation_label_iterate T_COMA label
	;


/********** Binding indication **********/

binding_indication
	: binding_indication_use_entity_aspect_optional
	  binding_indication_generic_map_aspect_optional
	  binding_indication_port_map_aspect_optional
	;

binding_indication_use_entity_aspect_optional
	: //empty
	| T_USE entity_aspect
	;

binding_indication_generic_map_aspect_optional
	: //empty
	| generic_map_aspect
	;

binding_indication_port_map_aspect_optional
	: //empty
	| port_map_aspect
	;

entity_aspect
	: T_ENTITY name
	| T_ENTITY name T_LPAREN identifier T_RPAREN
	| T_CONFIGURATION name
	| T_OPEN
	;

generic_map_aspect
	: T_GENERIC T_MAP T_LPAREN association_list T_RPAREN
	;

port_map_aspect
	: T_PORT T_MAP T_LPAREN association_list T_RPAREN
	;



/********************************** Disconnection specification **********************************/

disconnection_specification
	: T_DISCONNECT guarded_signal_specification T_AFTER expression T_SEMICOLON
	;

guarded_signal_specification
	: signal_list T_COLON type_mark
	;

signal_list
	: signal_list_signal_name_iterate
	| T_OTHERS
	| T_ALL
	;

signal_list_signal_name_iterate
	: name
	| signal_list_signal_name_iterate T_COMA name
	;





/*#####################################################
  #                       Names                       #
  #####################################################*/


/********************************** Names **********************************/

name
	: name_wrapped
	;

name_wrapped
	: name_wr
		{
			if (yychar == yyempty_) {
				yychar = driver.p_symtable->getToken(*$1, yyempty_);
			} else {
				std::cerr << "Error: The lookahead isn't empty. Cannot insert auxiliary token. Lookahead: " << yychar << "\n";
			}
		}
	;

name_wr
	: simple_name
		{$$ = $1;}
	| operator_symbol
		{$$ = $1;}
	| selected_name
		{$$ = p_empty_str;}
	| indexed_name
		{$$ = p_empty_str;}
	| slice_name
		{$$ = p_empty_str;}
	| attribute_name
		{$$ = $1;}
	;

prefix
	: name
	| function_call
	;



/********************************** Simple names **********************************/

simple_name_wrapped
	: identifier
		{
			if (yychar == yyempty_) {
				yychar = driver.p_symtable->getToken(*$1, yyempty_);
			} else {
				std::cerr << "Error: The lookahead isn't empty. Cannot insert auxiliary token. Lookahead: " << yychar << "\n";
			}
		}
	;

simple_name
	: simple_name_wrapped
	;



/********************************** Selected names **********************************/

selected_name
	: prefix T_DOT suffix
	;

suffix
	: simple_name
	| T_CHARACTER_LITERAL
	| operator_symbol
	| T_ALL
	;



/********************************** Indexed names **********************************/

indexed_name
	: prefix T_LPAREN indexed_name_expression_iterate T_RPAREN
	;

indexed_name_expression_iterate
	: expression
	| indexed_name_expression_iterate T_COMA expression
	;



/********************************** Slice names **********************************/

slice_name
	: prefix T_LPAREN discrete_range T_RPAREN
	;



/********************************** Attribute names **********************************/

attribute_name
	: attribute_name_wrapped
	;

attribute_name_wrapped
	: attribute_name_wr
		{
			if (yychar == yyempty_) {
				yychar = driver.p_symtable->getToken(*$1, yyempty_);
			} else {
				std::cerr << "Error: The lookahead isn't empty. Cannot insert auxiliary token. Lookahead: " << yychar << "\n";
			}
		}
	;

attribute_name_wr
	: prefix attribute_name_signature_optional T_APOSTROPHE attribute_designator attribute_name_expression_optional
		{$$ = $4;} // FIXME Tady absolutně nedokážu posoudit, která část by se měla při vyhledávání v ST brát v úvahu. V ST mohou být jen identifikátory, protože nic jiného se do ní nevkládá. Je tedy nesmysl v ní hledat nějaké apostrofy a podobné nesmysly. Prozatím tedy vezmeme v úvahu pouze attribude_designator, který se přepisuje na identifikátor. Vše ostatní budeme ignorovat.
	;

attribute_name_signature_optional
	: //empty
	| signature
	;

attribute_designator
	: simple_name
	;

attribute_name_expression_optional
	: //empty
	| T_LPAREN expression T_RPAREN
	;





/*#####################################################
  #                    Expressions                    #
  #####################################################*/


/********************************** Expressions **********************************/

expression
	: relation T_AND expression_and_relation_iterate
	| relation T_OR expression_or_relation_iterate
	| relation T_XOR expression_xor_relation_iterate
	| relation T_NAND relation
	| relation T_NOR relation
	| relation T_XNOR expression_xnor_relation_iterate
	| relation
	;

expression_and_relation_iterate
	: relation
	| expression_and_relation_iterate T_AND relation
	;

expression_or_relation_iterate
	: relation
	| expression_or_relation_iterate T_OR relation
	;

expression_xor_relation_iterate
	: relation
	| expression_xor_relation_iterate T_XOR relation
	;

expression_xnor_relation_iterate
	: relation
	| expression_xnor_relation_iterate T_XNOR relation
	;

relation
	: shift_expression
	| shift_expression relational_operator shift_expression
	;

shift_expression
	: simple_expression
	| simple_expression shift_operator simple_expression
	;

simple_expression
	: term simple_expression_adding_operator_term_iterate
	| sign term simple_expression_adding_operator_term_iterate
	;

simple_expression_adding_operator_term_iterate
	: //empty
	| simple_expression_adding_operator_term_iterate adding_operator term
	;

term
	: factor
	| term multiplying_operator factor
	;

factor
	: primary
	| primary T_DOUBLESTAR primary
	| T_ABS primary
	| T_NOT primary
	;

primary
	: name
	| literal
	| aggregate
	| function_call
	| qualified_expression
	| type_conversion
	| allocator
	| T_LPAREN expression T_RPAREN
	;



/********************************** Operators **********************************/

/*logical_operator		// Useless nonterminal?
	: T_AND
	| T_OR
	| T_NAND
	| T_NOR
	| T_XOR
	| T_XNOR
	;*/

relational_operator
	: T_EQUAL
	| T_INEQUALITY
	| T_LESS
	| T_LESS_THAN_OR_EQUAL
	| T_BIGGER
	| T_GREATER_THAN_OR_EQUAL
	;

shift_operator
	: T_SLL
	| T_SRL
	| T_SLA
	| T_SRA
	| T_ROL
	| T_ROR
	;

adding_operator
	: T_PLUS
	| T_MINUS
	| T_AMPERSAND
	;

sign
	: T_PLUS %prec T_UNARY_PLUS
	| T_MINUS %prec T_UNARY_MINUS
	;

multiplying_operator
	: T_STAR
	| T_SLASH
	| T_MOD
	| T_REM
	;

/*miscellaneous_operator		// Useless nonterminal?
	: T_DOUBLESTAR
	| T_ABS
	| T_NOT
	;*/



/********************************** Operands **********************************/

/********** Literals **********/

literal
	: numeric_literal
//	| enumeration_literal	// Nesmí se odkomentovat - viz další komentář!!!!!!
	| T_CHARACTER_LITERAL	// Zde bylo pravidlo pro přepis na enumeration_literal, ale ten se dále přepisuje na identifier nebo T_CHARACTER_LITERAL. Díky tomu vzniká R/R konflikt, kdy se identifier může redukovat na primary přes name i přes literal s naprosto stejným lookaheadem. Vzhledem k tomu, že literal se dále redukuje pouze na primary, bylo pravidlo pro přepis na enumerration_literal nahrazeno pravidlem pouze pro přepis na T_CHARACTER_LITERAL. Přepis na identifier je tak možný pouze přes T_NT name, čímž bylo odstraněno 50 R/R konfliktů a zároveň zůstal nedotčený T_NT enumeration_literal, který se vyskytuje ve více pravidlech.
//	Na literal se přepisuje jen "primary" a z toho lze přes name derivovat i identifier, takže tato změna neovlivní jazyk generovaný touto gramatikou, i když na první pohled by to mohlo svádět k doměnce, že se na idettifier má derivovat přes numeric_literal -> physical_literal, kde je name zablokováno pomocným terminálem.
//	| T_STRING_LITERAL	// Podobně jako v případě enumeration literal. T_STRING_LITERAL by opět bylo možné nejednoznačně redukovat na primary přes name -> operator_symbol -> T_STRING_LITERAL i přes literal...
	| T_BIT_STRING_LITERAL_B
	| T_BIT_STRING_LITERAL_O
	| T_BIT_STRING_LITERAL_X
	| T_NULL
	;

numeric_literal
	: T_ABSTRACT_LITERAL
	| physical_literal
	;


/********** Aggregates **********/

aggregate
	: T_LPAREN aggregate_element_association_iterate T_RPAREN
	;

aggregate_element_association_iterate
	: element_association
	| aggregate_element_association_iterate T_COMA element_association
	;

element_association
	: expression
	| choices T_ARROW expression
	;

choices
	: choice
	| choices T_PIPE choice
	;

choice
	: simple_expression
	| discrete_range
	| simple_name T__ELEMENT_
	| T_OTHERS
	;


/********** Function calls **********/

function_call
	: name T__FUNCTION_
	| name T__FUNCTION_ T_LPAREN actual_parameter_part T_RPAREN
	;

actual_parameter_part
	: association_list
	;


/********** Qualified expressions **********/

qualified_expression
	: type_mark T_APOSTROPHE T_LPAREN expression T_RPAREN
	| type_mark T_APOSTROPHE aggregate
	;


/********** Type conversions **********/

type_conversion
	: type_mark T_LPAREN expression T_RPAREN
	;


/********** Allocators **********/

allocator
	: T_NEW subtype_indication
	| T_NEW qualified_expression
	;





/*#####################################################
  #               Sequential statements               #
  #####################################################*/


sequence_of_statements
	: //empty
	| sequence_of_statements sequential_statement
	;

sequential_statement
	: wait_statement
	| assertion_statement
	| report_statement
	| signal_assignment_statement
	| variable_assignment_statement
	| procedure_call_statement
	| if_statement
	| case_statement
	| loop_statement
	| next_statement
	| exit_statement
	| return_statement
	| null_statement
	;



/********************************** Wait statement **********************************/

wait_statement
	: wait_statement_label_colon_optional T_WAIT wait_statement_sensitivity_clause_optional
	  wait_statement_condition_clause_optional wait_statement_timeout_clause_optional T_SEMICOLON
	;

wait_statement_label_colon_optional
	: //empty
	| label T_COLON
	;

wait_statement_sensitivity_clause_optional
	: //empty
	| sensitivity_clause
	;

wait_statement_condition_clause_optional
	: //empty
	| condition_clause
	;

wait_statement_timeout_clause_optional
	: //empty
	| timeout_clause
	;

sensitivity_clause
	: T_ON sensitivity_list
	;

sensitivity_list
	: name
	| sensitivity_list T_COMA name
	;

condition_clause
	: T_UNTIL condition
	;

condition
	: expression
	;

timeout_clause
	: T_FOR expression
	;



/********************************** Assertion statement **********************************/

assertion_statement
	: assertion_statement_label_colon_optional assertion T_SEMICOLON
	;

assertion_statement_label_colon_optional
	: //empty
	| label T_COLON
	;

assertion
	: T_ASSERT condition assertion_report_expression_optional assertion_severity_expression_optional
	;
assertion_report_expression_optional
	: //empty
	| T_REPORT expression
	;

assertion_severity_expression_optional
	: //empty
	| T_SEVERITY expression
	;



/********************************** Report statement **********************************/

report_statement
	: T_REPORT expression T_SEMICOLON
	| label T_COLON T_REPORT expression T_SEMICOLON
	| T_REPORT expression T_SEVERITY expression T_SEMICOLON
	| label T_COLON T_REPORT expression T_SEVERITY expression T_SEMICOLON
	;



/********************************** Signal assignment statement **********************************/


signal_assignment_statement
	: target T_LESS_THAN_OR_EQUAL waveform T_SEMICOLON
	| label T_COLON target T_LESS_THAN_OR_EQUAL waveform T_SEMICOLON
	| target T_LESS_THAN_OR_EQUAL delay_mechanism waveform T_SEMICOLON
	| label T_COLON target T_LESS_THAN_OR_EQUAL delay_mechanism waveform T_SEMICOLON
	;

delay_mechanism
	: T_TRANSPORT
	| T_INERTIAL
	| T_REJECT expression T_INERTIAL
	;

target
	: name
	| aggregate
	;

waveform
	: waveform_waveform_element_iterate
	| T_UNAFFECTED
	;

waveform_waveform_element_iterate
	: waveform_element
	| waveform_waveform_element_iterate T_COMA waveform_element
	;


/********** Updating a projected output waveform **********/

waveform_element
	: expression
	| expression T_AFTER expression
//	| T__NULL	// R/R konflikt s pravidlem literal -> _NULL - expression lze přímo přepsat na literal
//	| T__NULL T_AFTER expression
	;



/********************************** Variable assignment statement **********************************/

variable_assignment_statement
	: target T_VARIABLE_ASSIGNMENT expression T_SEMICOLON
	| label T_COLON target T_VARIABLE_ASSIGNMENT expression T_SEMICOLON
	;



/********************************** procedure call statement **********************************/

procedure_call_statement
	: procedure_call T_SEMICOLON
	| label T_COLON procedure_call T_SEMICOLON
	;

procedure_call
	: name
	| name T_LPAREN actual_parameter_part T_RPAREN
	;



/********************************** If statement **********************************/

if_statement
	: if_statement_label_colon_optional
	  T_IF condition T_THEN
	    sequence_of_statements
	  if_statement_elsif_iterate
	  if_statement_else_optional
	  T_END T_IF if_statement_label_optional T_SEMICOLON
	;

if_statement_label_colon_optional
	: //empty
	| label T_COLON
	;

if_statement_elsif_iterate
	: //empty
	| if_statement_elsif_iterate T_ELSIF condition T_THEN sequence_of_statements
	;

if_statement_else_optional
	: //empty
	| T_ELSE sequence_of_statements
	;

if_statement_label_optional
	: //empty
	| label
	;



/********************************** Case statement **********************************/

case_statement
	: case_statement_label_colon_optional
	  T_CASE expression T_IS
	    case_statement_case_statement_alternative_iterate
	  T_END T_CASE case_statement_label_optional T_SEMICOLON
	;

case_statement_label_colon_optional
	: //empty
	| label T_COLON
	;

case_statement_case_statement_alternative_iterate
	: case_statement_alternative
	| case_statement_case_statement_alternative_iterate case_statement_alternative
	;

case_statement_label_optional
	: //empty
	| label
	;

case_statement_alternative
	: T_WHEN choices T_ARROW sequence_of_statements
	;



/********************************** Loop statement **********************************/

loop_statement
	: loop_statement_label_colon_optional loop_statement_iteration_scheme_optional T_LOOP
	  sequence_of_statements T_END T_LOOP loop_statement_label_optional T_SEMICOLON
	;

loop_statement_label_colon_optional
	: //empty
	| label T_COLON
	;

loop_statement_iteration_scheme_optional
	: //empty
	| iteration_scheme
	;

loop_statement_label_optional
	: //empty
	| label
	;

iteration_scheme
	: T_WHILE condition
	| T_FOR parameter_specification
	;

parameter_specification
	: identifier T_IN discrete_range
	;



/********************************** Next statement **********************************/

next_statement
	: next_statement_label_colon_optional T_NEXT next_statement_label_optional
	  next_statement_when_condition_optional T_SEMICOLON
	;

next_statement_label_colon_optional
	: //empty
	| label T_COLON
	;

next_statement_label_optional
	: //empty
	| label
	;

next_statement_when_condition_optional
	: //empty
	| T_WHEN condition
	;



/********************************** Exit statement **********************************/

exit_statement
	: exit_statement_label_colon_optional T_EXIT exit_statement_label_optional
	  exit_statement_when_condition_optional T_SEMICOLON
	;

exit_statement_label_colon_optional
	: //empty
	| label T_COLON
	;

exit_statement_label_optional
	: //empty
	| label
	;

exit_statement_when_condition_optional
	: //empty
	| T_WHEN condition
	;



/********************************** Return statement **********************************/

return_statement
	: T_RETURN T_SEMICOLON
	| label T_COLON T_RETURN T_SEMICOLON
	| T_RETURN expression T_SEMICOLON
	| label T_COLON T_RETURN expression T_SEMICOLON
	;



/********************************** Null statement **********************************/

null_statement
	: T_NULL T_SEMICOLON
	| label T_COLON T_NULL T_SEMICOLON
	;





/*#####################################################
  #               Concurrent statements               #
  #####################################################*/



concurrent_statement
	: block_statement
	| process_statement
	| concurrent_procedure_call_statement
	| concurrent_assertion_statement
	| concurrent_signal_assignment_statement
	| component_instantiation_statement
	| generate_statement
	;



/********************************** Block statement **********************************/

block_statement
	: label T_COLON
	    T_BLOCK block_statement_expression_optional is_optional
	      block_header
	      block_declarative_part
	    T_BEGIN
		  block_statement_part
	    T_END T_BLOCK block_statement_label_optional T_SEMICOLON
	;

block_statement_expression_optional
	: //empty
	|	T_LPAREN expression T_RPAREN
	;

block_statement_label_optional
	: //empty
	|	label
	;

block_header
	: block_header_first_optional block_header_second_optional
	;

block_header_first_optional
 : // empty
	| generic_clause
	| generic_clause generic_map_aspect T_SEMICOLON
	;

block_header_second_optional
	: //empty
	| port_clause
	| port_clause port_map_aspect T_SEMICOLON
	;

block_declarative_part
	: //empty
	| block_declarative_part block_declarative_item
	;

block_statement_part
	: //empty
	| block_statement_part concurrent_statement
	;



/********************************** Process statement **********************************/

process_statement
	: process_statement_label_colon_optional
	    postponed_optional T_PROCESS process_statement_sensitivity_list_optional is_optional
	      process_declarative_part
	    T_BEGIN
	      process_statement_part
		T_END postponed_optional T_PROCESS process_statement_label_optional T_SEMICOLON
	;

process_statement_label_colon_optional
	: //empty
	| label T_COLON
	;

process_statement_sensitivity_list_optional
	: //empty
	| T_LPAREN sensitivity_list T_RPAREN
	;

process_statement_label_optional
	: //empty
	| label
	;

process_declarative_part
	: //empty
	| process_declarative_part process_declarative_item
	;

process_declarative_item
	: subprogram_declaration
	| subprogram_body
	| type_declaration
	| subtype_declaration
	| constant_declaration
	| variable_declaration
	| file_declaration
	| alias_declaration
	| attribute_declaration
	| attribute_specification
	| use_clause
	| group_template_declaration
	| group_declaration
	;

process_statement_part
	: //empty
	| process_statement_part sequential_statement
	;



/********************************** Concurrent procedure call statements **********************************/

concurrent_procedure_call_statement
	: postponed_optional procedure_call T_SEMICOLON
	| label T_COLON postponed_optional procedure_call T_SEMICOLON
	;



/********************************** Concurrent assertion statements **********************************/

concurrent_assertion_statement
	: postponed_optional assertion T_SEMICOLON
	| label T_COLON postponed_optional assertion T_SEMICOLON
	;



/********************************** Concurrent signal assignment statements **********************************/

concurrent_signal_assignment_statement
	: postponed_optional conditional_signal_assignment
	| label T_COLON postponed_optional conditional_signal_assignment
	| postponed_optional selected_signal_assignment
	| label T_COLON postponed_optional selected_signal_assignment
	;

options
	: //empty
	| T_GUARDED
	| delay_mechanism
	| T_GUARDED delay_mechanism
	;


/********** Conditional signal assignments **********/

conditional_signal_assignment
	: target T_LESS_THAN_OR_EQUAL options conditional_waveforms T_SEMICOLON
	;

conditional_waveforms
	: conditional_waveforms_iterate waveform T_WHEN condition
	| conditional_waveforms_iterate waveform
	;

conditional_waveforms_iterate
	: //empty
	| conditional_waveforms_iterate waveform T_WHEN condition T_ELSE
	;


/********** Selected signal assignments **********/

selected_signal_assignment
	: T_WITH expression T_SELECT target T_LESS_THAN_OR_EQUAL options selected_waveforms T_SEMICOLON
	;

selected_waveforms
	: waveform T_WHEN choices
	| selected_waveforms T_COMA waveform T_WHEN choices
	;



/********************************** Component instantiation statements **********************************/

component_instantiation_statement
	: label T_COLON
	    instantiated_unit
	      component_instantiation_statement_generic_map_aspect_optional
	      component_instantiation_statement_port_map_aspect_optional T_SEMICOLON
	;

component_instantiation_statement_generic_map_aspect_optional
	: //empty
	| generic_map_aspect
	;

component_instantiation_statement_port_map_aspect_optional
	: //empty
	| port_map_aspect
	;

instantiated_unit
	: name
	| T_COMPONENT name
	| T_ENTITY name
	| T_ENTITY name T_LPAREN identifier T_RPAREN
	| T_CONFIGURATION name
	;



/********************************** Generate statements **********************************/

generate_statement
	: label T_COLON
	    generation_scheme T_GENERATE
		generate_statement_block_declarative_item_begin_optional
	      generate_statement_concurrent_statement_iterate
	    T_END T_GENERATE generate_statement_label_optional T_SEMICOLON {driver.p_symtable->addToSet(std::symtable::s_generate_statement, *$1); $$ = 0;}	//TODO - zkontrolovat, upravit
	;

generate_statement_block_declarative_item_begin_optional
	: //empty
	| generate_statement_block_declarative_item_iterate T_BEGIN
	;

generate_statement_block_declarative_item_iterate
	: //empty
	| generate_statement_block_declarative_item_iterate block_declarative_item
	;

generate_statement_concurrent_statement_iterate
	: //empty
	| generate_statement_concurrent_statement_iterate concurrent_statement
	;

generate_statement_label_optional
	: //empty
	| label
	;

generation_scheme
	: T_FOR parameter_specification
	| T_IF condition
	;

label_wrapped
	: identifier
		{
			if (yychar == yyempty_) {
				yychar = driver.p_symtable->getToken(*$1, yyempty_);
			} else {
				std::cerr << "Error: The lookahead isn't empty. Cannot insert auxiliary token. Lookahead: " << yychar << "\n";
			}
		}
	;

label
	: label_wrapped
		{$$ = $1;}
	;





/*#####################################################
  #                Scope and visibility               #
  #####################################################*/


/********************************** Use clauses **********************************/

use_clause
	: T_USE selected_name selected_name_iterate T_SEMICOLON
	;

selected_name_iterate
	: //empty
	| selected_name_iterate T_COMA selected_name
	;





/*#####################################################
  #          Design units and their analysis          #
  #####################################################*/


/********************************** Design units **********************************/

design_file
	: design_unit
	| design_file design_unit
	;

design_unit
	: context_clause library_unit
	;

library_unit
	: primary_unit
	| secondary_unit
	;

primary_unit
	: entity_declaration
	| configuration_declaration
	| package_declaration
	;

secondary_unit
	: architecture_body
	| package_body
	;


/********************************** Design libraries **********************************/

library_clause
	: T_LIBRARY logical_name_list T_SEMICOLON
	;

logical_name_list
	: logical_name
	| logical_name_list T_COMA logical_name
	;

logical_name
	: identifier
	;


/********************************** Context clauses **********************************/

context_clause
	: //empty
	| context_clause context_item
	;

context_item
	: library_clause
	| use_clause
	;






/*#####################################################
  #                    Common rules                   #
  #####################################################*/

architecture_optional
	: //empty
	| T_ARCHITECTURE
	;

entity_optional
	: //empty
	| T_ENTITY
	;

simple_name_optional
	: //empty
	| simple_name
	;

configuration_optional
	: //empty
	| T_CONFIGURATION
	;

package_optional
	: //empty
	| T_PACKAGE
	;

is_optional
	: //empty
	| T_IS
	;

postponed_optional
	: //empty
	| T_POSTPONED
	;

identifier
	: T_BASIC_IDENTIFIER
		{$$ = $1;}
	| T_EXTENDED_IDENTIFIER
		{$$ = $1;}
	;



%%

void yy::vhdl_parser::error (const yy::vhdl_parser::location_type& l, const std::string& m)
{
  driver.error (l, m);
}
