#include "symtable.h"
#include "parser.tab.hh"

namespace std {


symtable::symtable()
{
	name_type.insert("integer");
	name_type.insert("time");
	name_type.insert("real");
	name_type.insert("string");		// je definován v balíku standard
	name_type.insert("bit_vector");

	name_type.insert("std_logic");
	name_type.insert("std_logic_vector");
}


symtable::~symtable()
{
}

void symtable::addToSet(sets _set, string what)
{
	getSet(_set).insert(what);
}


void symtable::addSetToSet(sets _set, set<string> * what)
{
	getSet(_set).insert(what->begin(), what->end());
}


bool symtable::isInSet(sets _set, string what)
{
	return getSet(_set).count(what);
}


set< string > & std::symtable::getSet(sets _set)
{
	switch (_set) {
		case s_architecture:
			return name_architecture;
			break;
		case s_range:
			return name_range;
			break;
		case s_unit:
			return name_unit;
			break;
		case s_type:
			return name_type;
			break;
		case s_subtype:
			return name_subtype;
			break;
		case s_function:
			return name_function;
			break;
		case s_signal:
			return name_signal;
			break;
		case s_variable:
			return name_variable;
			break;
		case s_file:
			return name_file;
			break;
		case s_generic:
			return name_generic;
			break;
		case s_port:
			return name_port;
			break;
		case s_parameter:
			return name_parameter;
			break;
		case s_element:
			return simple_name_element;
			break;
		case s_generate_statement:
			return label_generate_statement;
			break;
	}
}


int symtable::getToken(string str, int fall_back)
{
	cerr << "Teď se pokouším vkládat... Řetězec: " << str;
	if (isInSet(s_generic, str)) return yy::vhdl_parser::token::T__GENERIC_;
	if (isInSet(s_port, str)) return yy::vhdl_parser::token::T__PORT_;
	if (isInSet(s_parameter, str)) return yy::vhdl_parser::token::T__PARAMETER_;
	if (isInSet(s_function, str)) return yy::vhdl_parser::token::T__FUNCTION_;
	if (isInSet(s_signal, str)) return yy::vhdl_parser::token::T__SIGNAL_;
	if (isInSet(s_variable, str)) return yy::vhdl_parser::token::T__VARIABLE_;
	if (isInSet(s_file, str)) return yy::vhdl_parser::token::T__FILE_;
	if (isInSet(s_architecture, str)) return yy::vhdl_parser::token::T__ARCHITECTURE_;
	if (isInSet(s_range, str)) return yy::vhdl_parser::token::T__RANGE_;
	if (isInSet(s_unit, str)) return yy::vhdl_parser::token::T__UNIT_;
	if (isInSet(s_type, str)) return yy::vhdl_parser::token::T__TYPE_;
	if (isInSet(s_subtype, str)) return yy::vhdl_parser::token::T__SUBTYPE_;
	if (isInSet(s_element, str)) return yy::vhdl_parser::token::T__ELEMENT_;
	if (isInSet(s_generate_statement, str)) return yy::vhdl_parser::token::T__GENERATE_STATEMENT_;
	return fall_back;
}


}

